@@ -409,10 +409,17 @@ xfdesktop_file_utils_get_icon(GIcon *icon,
 
 
     if(pix_theme) {
+        GdkPixbuf *tmp;
         /* we can't edit thsese icons */
-        pix = gdk_pixbuf_copy(pix_theme);
+        tmp = gdk_pixbuf_copy(pix_theme);
+
+        /* ensure icons are within our size requirements since
+         * gtk_icon_theme_lookup_by_gicon isn't exact */
+        pix = exo_gdk_pixbuf_scale_down(tmp, TRUE, width, height);
+
+        g_object_unref(G_OBJECT(tmp));
         g_object_unref(G_OBJECT(pix_theme));
-        pix_theme = NULL;
+        pix_theme = tmp = NULL;
     }
 
     /* fallback */
@@ -441,7 +448,7 @@ xfdesktop_file_utils_get_icon(GIcon *icon,
 static void
 xfdesktop_file_utils_add_emblems(GdkPixbuf *pix, GList *emblems)
 {
-    GdkPixbuf *emblem_pix;
+    GdkPixbuf *emblem_pix = NULL;
     gint max_emblems;
     gint pix_width, pix_height;
     gint emblem_size;
@@ -774,54 +781,38 @@ xfdesktop_file_utils_bulk_rename(GFile *working_directory,
     }
 }
 
+static void
+delete_files(gpointer data, gpointer user_data)
+{
+    if(data == NULL)
+        return;
+
+    if(!G_IS_FILE(data))
+        return;
+
+    g_file_delete_async(G_FILE(data), G_PRIORITY_DEFAULT, NULL, NULL, NULL);
+}
+
+static void
+trash_files(gpointer data, gpointer user_data)
+{
+    if(data == NULL)
+        return;
+
+    if(!G_IS_FILE(data))
+        return;
+
+    g_file_trash_async(G_FILE(data), G_PRIORITY_DEFAULT, NULL, NULL, NULL);
+}
+
 void
 xfdesktop_file_utils_unlink_files(GList *files,
                                   GdkScreen *screen,
                                   GtkWindow *parent)
 {
-    DBusGProxy *fileman_proxy;
-
     g_return_if_fail(files != NULL && G_IS_FILE(files->data));
-    g_return_if_fail(GDK_IS_SCREEN(screen) || GTK_IS_WINDOW(parent));
 
-    if(!screen)
-        screen = gtk_widget_get_screen(GTK_WIDGET(parent));
-
-    fileman_proxy = xfdesktop_file_utils_peek_filemanager_proxy();
-    if(fileman_proxy) {
-        guint nfiles = g_list_length(files);
-        gchar **uris = g_new0(gchar *, nfiles+1);
-        gchar *display_name = gdk_screen_make_display_name(screen);
-        gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
-        GList *lp;
-        gint n;
-
-        /* convert GFile list into an array of URIs */
-        for(n = 0, lp = files; lp != NULL; ++n, lp = lp->next)
-            uris[n] = g_file_get_uri(lp->data);
-        uris[n] = NULL;
-
-        xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
-
-        xfdesktop_file_manager_proxy_unlink_files_async(fileman_proxy,
-                                                        NULL, (const gchar **)uris,
-                                                        display_name, startup_id,
-                                                        (xfdesktop_file_manager_proxy_unlink_files_reply)xfdesktop_file_utils_async_cb,
-                                                        parent);
-
-        xfdesktop_file_utils_set_window_cursor(parent, GDK_LEFT_PTR);
-
-        g_free(startup_id);
-        g_strfreev(uris);
-        g_free(display_name);
-    } else {
-        xfce_message_dialog(parent,
-                            _("Delete Error"), GTK_STOCK_DIALOG_ERROR,
-                            _("The selected files could not be deleted"),
-                            _("This feature requires a file manager service to "
-                              "be present (such as the one supplied by Thunar)."),
-                            GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
-    }
+    g_list_foreach(files, (GFunc)delete_files, NULL);
 }
 
 void
@@ -829,49 +820,9 @@ xfdesktop_file_utils_trash_files(GList *files,
                                  GdkScreen *screen,
                                  GtkWindow *parent)
 {
-    DBusGProxy *trash_proxy;
-
     g_return_if_fail(files != NULL && G_IS_FILE(files->data));
-    g_return_if_fail(GDK_IS_SCREEN(screen) || GTK_IS_WINDOW(parent));
 
-    if(!screen)
-        screen = gtk_widget_get_screen(GTK_WIDGET(parent));
-
-    trash_proxy = xfdesktop_file_utils_peek_trash_proxy();
-    if(trash_proxy) {
-        guint nfiles = g_list_length(files);
-        gchar **uris = g_new0(gchar *, nfiles+1);
-        gchar *display_name = gdk_screen_make_display_name(screen);
-        gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
-        GList *lp;
-        gint n;
-
-        /* convert GFile list into an array of URIs */
-        for(n = 0, lp = files; lp != NULL; ++n, lp = lp->next)
-            uris[n] = g_file_get_uri(lp->data);
-        uris[n] = NULL;
-
-        xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
-
-        xfdesktop_trash_proxy_move_to_trash_async(trash_proxy,
-                                                  (const gchar **)uris,
-                                                  display_name, startup_id,
-                                                  (xfdesktop_trash_proxy_move_to_trash_reply)xfdesktop_file_utils_async_cb,
-                                                  parent);
-
-        xfdesktop_file_utils_set_window_cursor(parent, GDK_LEFT_PTR);
-
-        g_free(startup_id);
-        g_strfreev(uris);
-        g_free(display_name);
-    } else {
-        xfce_message_dialog(parent,
-                            _("Trash Error"), GTK_STOCK_DIALOG_ERROR,
-                            _("The selected files could not be moved to the trash"),
-                            _("This feature requires a trash service to "
-                              "be present (such as the one supplied by Thunar)."),
-                            GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
-    }
+    g_list_foreach(files, (GFunc)trash_files, NULL);
 }
 
 void
